<?php

/**
 * These are used to construct the CVSLib_Error objects
 * to identify problem classes
 */

define('CVSLIB_INTERNAL_ERROR', 1);
define('CVSLIB_NOT_FOUND', 2);
define('CVSLIB_PERMISSION_DENIED', 3);

/**
 * Every class in this package has an id() function 
 * which returns what it is, to allow the calling function
 * to figure out whether it is an error or a data return type
 */

define('CVSLIB_ERROR', 4);
define('CVSLIB_CHECKOUT', 5);
define('CVSLIB_REPOSITORY', 6);
define('CVSLIB_DIRECTORY', 7);
define('CVSLIB_FILE', 8);
define('CVSLIB_ANNOTATE', 9);

define('CVSLIB_LOG_INIT', 10);
define('CVSLIB_LOG_FILENAME', 11);
define('CVSLIB_LOG_REVISION', 12);
define('CVSLIB_LOG_INFO', 13);
define('CVSLIB_LOG_DATE', 14);
define('CVSLIB_LOG_BRANCHES', 15);
define('CVSLIB_LOG_DONE', 16);

define('CVSLIB_LOG_FULL', 0);
define('CVSLIB_LOG_QUICK', 1);

define('CVSLIB_ATTIC_HIDE', 0);
define('CVSLIB_ATTIC_SHOW', 1);

/**
 * Sorting options 
 */

define('CVSLIB_SORT_NONE', 0);        // don't sort
define('CVSLIB_SORT_AGE', 1);         // sort by age
define('CVSLIB_SORT_NAME', 2);        // sort by filename
define('CVSLIB_SORT_REV', 3);         // sort by revision number
define('CVSLIB_SORT_AUTHOR', 4);      // sort by author name

define('CVSLIB_SORT_ASCENDING', 0);   // ascending order
define('CVSLIB_SORT_DESCENDING', 1);  // descending order

/**
 * Diff options
 */

define('CVSLIB_DIFF_CONTEXT', 0);
define('CVSLIB_DIFF_UNIFIED', 1);
define('CVSLIB_DIFF_COLUMN',  2);
define('CVSLIB_DIFF_ED',      3);

define('CVSLIB_DIFF_HEADER', 'header');
define('CVSLIB_DIFF_ADD', 'add');
define('CVSLIB_DIFF_EMPTY', 'empty');
define('CVSLIB_DIFF_DUMP', 'dump');
define('CVSLIB_DIFF_REMOVE', 'remove');
define('CVSLIB_DIFF_CHANGE', 'change');

/**
 * Babel Base Class.
 *
 * $Horde: babel/lib/Babel.php,v 1.5 2004/10/19 00:45:22 chuck Exp $
 *
 * @author Jon Parise <jon@horde.org>
 * @version $Revision: 1.5 $
 * @package babel
 */
class Babel {

    function fixString($string, $lang)
    {
        /* TODO move these mappings to locale.php */
        switch($lang) {
        case 'fr_FR':
        default:
            $fix_s = array("\n", "\r", '&quot;', '&apos;', '&lt;', '&gt;', '&amp;');
            $fix_t = array(" ", "",  '\\"', "'", '<', '>', '&');
            $new = str_replace($fix_s, $fix_t, htmlEntities($string));
        }

        return $new;
    }

    /**
     * Generate a URL that links into Babel.
     * @param script Name of the Babel script to link into
     * @param uri Any PATH_INFO portion that should be included
     * @param args Key/value pair of any GET parameters to append
     * @param anchor Anchor entity name
     */
    function url($script, $uri='', $args = array(), $anchor='')
    {
        global $registry, $conf;
     
        $url = $registry->get('webroot') . '/' . $script . '.php';
        $uri = rawurlencode($uri);
     
        $arglist = $args;
     
        if (!empty($uri)) {
            $url .= '/' . $uri;
        }
     
        $argarr = array();
        foreach ($arglist as $key => $val) {
            if (!empty($val)) {
                $val = htmlspecialchars($val);
                $argarr[] = "$key=$val";
            }
        }

        if (sizeof($argarr) > 0) {
            $url = "$url?" . implode('&', $argarr);
            $glue = '&';
        } else {
            $glue = '?';
        }
     
        if (!empty($anchor)) {
            $url .= "#$anchor";
        }
     
        $url = preg_replace('|/\?|', '?', $url);
        $url = preg_replace('|%2F|', '/', $url);
        $url = preg_replace('|/+|', '/', $url);
        $url = str_replace('&', '&amp;', $url);
     
        return $url;
    }

    function makeSQLInsert($array) {
        if (is_Array($array)) {
            unset($V);
            unset($F);
            foreach ($array as $f=>$v) {
                $F[] = $f;
                if (is_String($v) || empty($v)) {
                    if ( (substr($v,0,2)=='%%') || (substr($v,0,1)!='%') ) {
                        if ( substr($v,0,2)=='%%' ) {
                            $v = substr($v,1);
                        }
                        if (get_magic_quotes_runtime()) {
                            $V[] = "'" . $v . "'";
                        } else {
                            $V[] = "'" . addslashes($v) . "'";
                        }
                    } else {
                        $V[] = substr($v,1);
                    }
                } else {
                    $V[] = $v;
                }
            }
            $sql = sprintf("(%s) VALUES (%s)", join(",\n",$F), join (",\n", $V) );
        } else {
            $sql = false;
        }
        return $sql;
    }
}

?>
