<?php
/**
 * @version     $Id: cvs_functions.php,v 1.12.2.2 2002/02/08 20:30:42 cmv Exp $
 * @copyright   2000, easyDNS Technologies Inc.
 */

function getAllFiles($list, $where, $showattic=false) {
	global $CVS, $VALID_EXTENSIONS, $VALID_MODULES, $WORKING_BRANCH;

	if ($where) {
		$D = $CVS->querydir($where);
	} else {
		$D = $CVS->querydir('/');
	}

	$attic = $showattic ? CVSLIB_ATTIC_SHOW : CVSLIB_ATTIC_HIDE;

	$D->browseDir(CVSLIB_LOG_QUICK, $attic);
	$D->applySort(CVSLIB_SORT_NAME, CVSLIB_SORT_ASCENDING);

	$dirs = $D->queryDirList();

	foreach ($dirs as $dir) {

		if (!$where && !in_array($dir, $VALID_MODULES)) {
			continue;
		}

		$data = array();
		$data['file']			= '';
		$data['cvs_revision']	= '';
		$data['attic']			= '';
		$data['ext']			= '';
		if ($where) {
			$data['path']		= $CVS->cvsRoot() . '/' . $where . '/' . $dir ;
		} else {
			$data['path']		= $CVS->cvsRoot() . '/' . $dir ;
		}

		array_push ($GLOBALS[$list], $data );

		if ($dir!='CVSROOT') {
			if ($where) {
				getAllFiles($list, $where.'/'.$dir, $showattic );
			} else {
				getAllFiles($list, $dir, $showattic );
			}
		}
	}

	$files = $D->queryFileList($attic);
	foreach ($files as $file) {
		$ext = $file->getextension();
		if (in_array($ext, $VALID_EXTENSIONS)) {
			$filename = preg_replace('/,v$/', '', $file->queryfullpath() );

			$temp = explode('/', $filename );

			$data = array();
			$data['file']			= array_pop($temp);
			$data['path']			= join('/', $temp);
			$data['branch']			= $WORKING_BRANCH;
			if ($WORKING_BRANCH!='head') {
				$key = array_search($WORKING_BRANCH, $file->branches, true);
				if ($key === false) {
					continue;			/* this file isn't in the branch, so skip it */
				} else {
					$data['cvs_revision']	= $key;
				}
			} else {
				$data['cvs_revision']	= $file->queryrevision();
			}
			$data['attic']			= $file->isAtticFile();
			$data['ext']			= $ext;

			array_push ($GLOBALS[$list], $data );

		}
	}
}

function findNewStrings($filename, $rev) {
	global $CVS;

	$results = array();

	/* kludge for branch files that haven't been changed since head */
	$rev_parts = explode('.',$rev);
	if (count($rev_parts)%2) {			/* odd number of parts */
		array_pop($rev_parts);
		$rev = join('.', $rev_parts);
	}

	$co = CVSLib_Checkout::get($CVS, $filename, $rev);

	if (is_resource($co)) {
		$string = '';
		while (!feof($co)) {
			$string .= fread($co, 10000);
		}
	} else {
		return array();
	}

	# build the regex to find strings:
	# 1) double quoted strings
	$DOUBLE = '"((\\\\.|[^"\\\\])*)"';

	# 2) single quoted strings
	$SINGLE = '\'((\\\\.|[^\'\\\\])*)\'';

	$BLOCK = "(($DOUBLE|$SINGLE)(\s*\.\s*)?)+";


	$COMMENT = '\/\*\s_\{\s*(.*?)\s*\}\s\*\/';

	if (preg_match_all("/_\(\s*($DOUBLE|$SINGLE|$BLOCK)\s*\)/sm", $string, $matches )) {

		$part = $string;
		$offset = 1;

		foreach ($matches[1] as $match) {

			list($before, $after) = explode($match, $part, 2);

			$lnum = $offset + strlen($before) - strlen( str_replace("\n", "", $before) );

			$test = @eval('if ($x = ' . $match . ') { return true; }');
			if ($test) {

				$x = str_replace("\n", '\\n', $x);
				$x = str_replace("\r", '\\r', $x);
				$x = str_replace("\t", '\\t', $x);
				$x = str_replace("\0", '\\0', $x);

				$temp = array(
					'string'	=> $x,
					'line_no'	=> $lnum,
					'checksum'	=> md5($x)
				);

				if (strstr($after, '_(')) {
					list($comment, $junk) = explode('_(', $after, 2);
					if ($comment) {
						if (preg_match("/$COMMENT/sm", $comment, $m)) {
							$temp['comment'] = trim($m[1]);
						}
					}
				}

			} else {

				$temp = array(
					'string'	=> '',
					'line_no'	=> $lnum,
					'checksum'	=> '',
					'error'		=> 'Parse error in ' . $filename . ', near line ' . $lnum
				);

			}

			array_push($results, $temp);

			$part = $after;
			$offset = $lnum;

		}

	}

	return $results;

}


function loadStrings($file) {
	global $DB;

	if (is_int($file)) {
		$query = 'SELECT * FROM strings WHERE page_id='. $file;
	} else {
		$query = 'SELECT s.* FROM pages p LEFT JOIN strings s USING (page_id) WHERE ' .
			 'CONCAT(p.path,'/',p.file)='. $file;
	}
	$query .= ' ORDER BY line_no';

	$q = $DB->query($query);

	$results = array();

	while ($r = $q->fetchRow($q)) {
		array_push($results, $r);
	}

	return $results;

}


function niceString($string, $maxlen=80)
{
	$ellipses = ' &#133; ';

	if (strlen($string) < $maxlen) {
		return HTMLSpecialChars($string);
	}

	$first = ltrim(substr($string, 0, ($maxlen/4)*3));
	$last = rtrim(substr($string, -($maxlen/4)));

	if (strrchr($first, ' ')) {
		$first = trim(substr($first, 0, -(strlen(strrchr($first, ' '))) ) );
	}

	if (strstr($last, ' ')!==false) {
		$last = trim(strstr($last, ' '));
	}

	return '<SPAN TITLE="' . HTMLSpecialChars($string) . '">' .
		HTMLSpecialChars($first) . $ellipses . HTMLSpecialChars($last) .
		'</SPAN>';

}

function is_CVS_error($obj)
{
	return is_a($obj, 'CVSLib_Error');
}
